# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.channel_v1.types import common
from google.cloud.channel_v1.types import products
from google.protobuf import timestamp_pb2  # type: ignore
from google.type import money_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.channel.v1",
    manifest={
        "PromotionalOrderType",
        "PaymentPlan",
        "PaymentType",
        "ResourceType",
        "PeriodType",
        "Offer",
        "ParameterDefinition",
        "Constraints",
        "CustomerConstraints",
        "Plan",
        "PriceByResource",
        "Price",
        "PricePhase",
        "PriceTier",
        "Period",
    },
)


class PromotionalOrderType(proto.Enum):
    r"""Constraints type for Promotional offers."""
    PROMOTIONAL_TYPE_UNSPECIFIED = 0
    NEW_UPGRADE = 1
    TRANSFER = 2
    PROMOTION_SWITCH = 3


class PaymentPlan(proto.Enum):
    r"""Describes how the reseller will be billed."""
    PAYMENT_PLAN_UNSPECIFIED = 0
    COMMITMENT = 1
    FLEXIBLE = 2
    FREE = 3
    TRIAL = 4
    OFFLINE = 5


class PaymentType(proto.Enum):
    r"""Specifies when the payment needs to happen."""
    PAYMENT_TYPE_UNSPECIFIED = 0
    PREPAY = 1
    POSTPAY = 2


class ResourceType(proto.Enum):
    r"""Represents the type for a monetizable resource(any entity on
    which billing happens). For example, this could be MINUTES for
    Google Voice and GB for Google Drive. One SKU can map to
    multiple monetizable resources.
    """
    RESOURCE_TYPE_UNSPECIFIED = 0
    SEAT = 1
    MAU = 2
    GB = 3
    LICENSED_USER = 4
    MINUTES = 5
    IAAS_USAGE = 6
    SUBSCRIPTION = 7


class PeriodType(proto.Enum):
    r"""Period Type."""
    PERIOD_TYPE_UNSPECIFIED = 0
    DAY = 1
    MONTH = 2
    YEAR = 3


class Offer(proto.Message):
    r"""Represents an offer made to resellers for purchase. An offer is
    associated with a [Sku][google.cloud.channel.v1.Sku], has a plan for
    payment, a price, and defines the constraints for buying.

    Attributes:
        name (str):
            Resource Name of the Offer. Format:
            accounts/{account_id}/offers/{offer_id}
        marketing_info (google.cloud.channel_v1.types.MarketingInfo):
            Marketing information for the Offer.
        sku (google.cloud.channel_v1.types.Sku):
            SKU the offer is associated with.
        plan (google.cloud.channel_v1.types.Plan):
            Describes the payment plan for the Offer.
        constraints (google.cloud.channel_v1.types.Constraints):
            Constraints on transacting the Offer.
        price_by_resources (Sequence[google.cloud.channel_v1.types.PriceByResource]):
            Price for each monetizable resource type.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Start of the Offer validity time.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. End of the Offer validity time.
        parameter_definitions (Sequence[google.cloud.channel_v1.types.ParameterDefinition]):
            Parameters required to use current Offer to
            purchase.
    """

    name = proto.Field(proto.STRING, number=1,)
    marketing_info = proto.Field(
        proto.MESSAGE, number=2, message=products.MarketingInfo,
    )
    sku = proto.Field(proto.MESSAGE, number=3, message=products.Sku,)
    plan = proto.Field(proto.MESSAGE, number=4, message="Plan",)
    constraints = proto.Field(proto.MESSAGE, number=5, message="Constraints",)
    price_by_resources = proto.RepeatedField(
        proto.MESSAGE, number=6, message="PriceByResource",
    )
    start_time = proto.Field(proto.MESSAGE, number=7, message=timestamp_pb2.Timestamp,)
    end_time = proto.Field(proto.MESSAGE, number=8, message=timestamp_pb2.Timestamp,)
    parameter_definitions = proto.RepeatedField(
        proto.MESSAGE, number=9, message="ParameterDefinition",
    )


class ParameterDefinition(proto.Message):
    r"""Parameter's definition. Specifies what parameter is required
    to use the current Offer to purchase.

    Attributes:
        name (str):
            Name of the parameter.
        parameter_type (google.cloud.channel_v1.types.ParameterDefinition.ParameterType):
            Data type of the parameter. Minimal value,
            Maximum value and allowed values will use
            specified data type here.
        min_value (google.cloud.channel_v1.types.Value):
            Minimal value of the parameter, if
            applicable. Inclusive. For example, minimal
            commitment when purchasing Anthos is 0.01.
            Applicable to INT64 and DOUBLE parameter types.
        max_value (google.cloud.channel_v1.types.Value):
            Maximum value of the parameter, if
            applicable. Inclusive. For example, maximum
            seats when purchasing Google Workspace Business
            Standard. Applicable to INT64 and DOUBLE
            parameter types.
        allowed_values (Sequence[google.cloud.channel_v1.types.Value]):
            If not empty, parameter values must be drawn from this list.
            For example, [us-west1, us-west2, ...] Applicable to STRING
            parameter type.
        optional (bool):
            If set to true, parameter is optional to
            purchase this Offer.
    """

    class ParameterType(proto.Enum):
        r"""Data type of the parameter."""
        PARAMETER_TYPE_UNSPECIFIED = 0
        INT64 = 1
        STRING = 2
        DOUBLE = 3

    name = proto.Field(proto.STRING, number=1,)
    parameter_type = proto.Field(proto.ENUM, number=2, enum=ParameterType,)
    min_value = proto.Field(proto.MESSAGE, number=3, message=common.Value,)
    max_value = proto.Field(proto.MESSAGE, number=4, message=common.Value,)
    allowed_values = proto.RepeatedField(proto.MESSAGE, number=5, message=common.Value,)
    optional = proto.Field(proto.BOOL, number=6,)


class Constraints(proto.Message):
    r"""Represents the constraints for buying the Offer.
    Attributes:
        customer_constraints (google.cloud.channel_v1.types.CustomerConstraints):
            Represents constraints required to purchase
            the Offer for a customer.
    """

    customer_constraints = proto.Field(
        proto.MESSAGE, number=1, message="CustomerConstraints",
    )


class CustomerConstraints(proto.Message):
    r"""Represents constraints required to purchase the Offer for a
    customer.

    Attributes:
        allowed_regions (Sequence[str]):
            Allowed geographical regions of the customer.
        allowed_customer_types (Sequence[google.cloud.channel_v1.types.CloudIdentityInfo.CustomerType]):
            Allowed Customer Type.
        promotional_order_types (Sequence[google.cloud.channel_v1.types.PromotionalOrderType]):
            Allowed Promotional Order Type. Present for
            Promotional offers.
    """

    allowed_regions = proto.RepeatedField(proto.STRING, number=1,)
    allowed_customer_types = proto.RepeatedField(
        proto.ENUM, number=2, enum=common.CloudIdentityInfo.CustomerType,
    )
    promotional_order_types = proto.RepeatedField(
        proto.ENUM, number=3, enum="PromotionalOrderType",
    )


class Plan(proto.Message):
    r"""The payment plan for the Offer. Describes how to make a
    payment.

    Attributes:
        payment_plan (google.cloud.channel_v1.types.PaymentPlan):
            Describes how a reseller will be billed.
        payment_type (google.cloud.channel_v1.types.PaymentType):
            Specifies when the payment needs to happen.
        payment_cycle (google.cloud.channel_v1.types.Period):
            Describes how frequently the reseller will be
            billed, such as once per month.
        trial_period (google.cloud.channel_v1.types.Period):
            Present for Offers with a trial period.
            For trial-only Offers, a paid service needs to
            start before the trial period ends for continued
            service.
            For Regular Offers with a trial period, the
            regular pricing goes into effect when trial
            period ends, or if paid service is started
            before the end of the trial period.
        billing_account (str):
            Reseller Billing account to charge after an
            offer transaction. Only present for Google Cloud
            Platform offers.
    """

    payment_plan = proto.Field(proto.ENUM, number=1, enum="PaymentPlan",)
    payment_type = proto.Field(proto.ENUM, number=2, enum="PaymentType",)
    payment_cycle = proto.Field(proto.MESSAGE, number=3, message="Period",)
    trial_period = proto.Field(proto.MESSAGE, number=4, message="Period",)
    billing_account = proto.Field(proto.STRING, number=5,)


class PriceByResource(proto.Message):
    r"""Represents price by resource type.
    Attributes:
        resource_type (google.cloud.channel_v1.types.ResourceType):
            Resource Type. Example: SEAT
        price (google.cloud.channel_v1.types.Price):
            Price of the Offer. Present if there are no
            price phases.
        price_phases (Sequence[google.cloud.channel_v1.types.PricePhase]):
            Specifies the price by time range.
    """

    resource_type = proto.Field(proto.ENUM, number=1, enum="ResourceType",)
    price = proto.Field(proto.MESSAGE, number=2, message="Price",)
    price_phases = proto.RepeatedField(proto.MESSAGE, number=3, message="PricePhase",)


class Price(proto.Message):
    r"""Represents the price of the Offer.
    Attributes:
        base_price (google.type.money_pb2.Money):
            Base price.
        discount (float):
            Discount percentage, represented as decimal.
            For example, a 20% discount will be represent as
            0.2.
        effective_price (google.type.money_pb2.Money):
            Effective Price after applying the discounts.
        external_price_uri (str):
            Link to external price list, such as link to
            Google Voice rate card.
    """

    base_price = proto.Field(proto.MESSAGE, number=1, message=money_pb2.Money,)
    discount = proto.Field(proto.DOUBLE, number=2,)
    effective_price = proto.Field(proto.MESSAGE, number=3, message=money_pb2.Money,)
    external_price_uri = proto.Field(proto.STRING, number=4,)


class PricePhase(proto.Message):
    r"""Specifies the price by the duration of months.
    For example, a 20% discount for the first six months, then a 10%
    discount starting on the seventh month.

    Attributes:
        period_type (google.cloud.channel_v1.types.PeriodType):
            Defines the phase period type.
        first_period (int):
            Defines first period for the phase.
        last_period (int):
            Defines first period for the phase.
        price (google.cloud.channel_v1.types.Price):
            Price of the phase. Present if there are no
            price tiers.
        price_tiers (Sequence[google.cloud.channel_v1.types.PriceTier]):
            Price by the resource tiers.
    """

    period_type = proto.Field(proto.ENUM, number=1, enum="PeriodType",)
    first_period = proto.Field(proto.INT32, number=2,)
    last_period = proto.Field(proto.INT32, number=3,)
    price = proto.Field(proto.MESSAGE, number=4, message="Price",)
    price_tiers = proto.RepeatedField(proto.MESSAGE, number=5, message="PriceTier",)


class PriceTier(proto.Message):
    r"""Defines price at resource tier level. For example, an offer with
    following definition :

    -  Tier 1: Provide 25% discount for all seats between 1 and 25.
    -  Tier 2: Provide 10% discount for all seats between 26 and 100.
    -  Tier 3: Provide flat 15% discount for all seats above 100.

    Each of these tiers is represented as a PriceTier.

    Attributes:
        first_resource (int):
            First resource for which the tier price
            applies.
        last_resource (int):
            Last resource for which the tier price
            applies.
        price (google.cloud.channel_v1.types.Price):
            Price of the tier.
    """

    first_resource = proto.Field(proto.INT32, number=1,)
    last_resource = proto.Field(proto.INT32, number=2,)
    price = proto.Field(proto.MESSAGE, number=3, message="Price",)


class Period(proto.Message):
    r"""Represents period in days/months/years.
    Attributes:
        duration (int):
            Total duration of Period Type defined.
        period_type (google.cloud.channel_v1.types.PeriodType):
            Period Type.
    """

    duration = proto.Field(proto.INT32, number=1,)
    period_type = proto.Field(proto.ENUM, number=2, enum="PeriodType",)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.protobuf import any_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.channel.v1",
    manifest={"EduData", "CloudIdentityInfo", "Value", "AdminUser",},
)


class EduData(proto.Message):
    r"""Required Edu Attributes
    Attributes:
        institute_type (google.cloud.channel_v1.types.EduData.InstituteType):
            Designated institute type of customer.
        institute_size (google.cloud.channel_v1.types.EduData.InstituteSize):
            Size of the institute.
        website (str):
            Web address for the edu customer's
            institution.
    """

    class InstituteType(proto.Enum):
        r"""Enum to specify the institute type."""
        INSTITUTE_TYPE_UNSPECIFIED = 0
        K12 = 1
        UNIVERSITY = 2

    class InstituteSize(proto.Enum):
        r"""Number of students and staff the institute has."""
        INSTITUTE_SIZE_UNSPECIFIED = 0
        SIZE_1_100 = 1
        SIZE_101_500 = 2
        SIZE_501_1000 = 3
        SIZE_1001_2000 = 4
        SIZE_2001_5000 = 5
        SIZE_5001_10000 = 6
        SIZE_10001_OR_MORE = 7

    institute_type = proto.Field(proto.ENUM, number=1, enum=InstituteType,)
    institute_size = proto.Field(proto.ENUM, number=2, enum=InstituteSize,)
    website = proto.Field(proto.STRING, number=3,)


class CloudIdentityInfo(proto.Message):
    r"""Cloud Identity information for the Cloud Channel Customer.
    Attributes:
        customer_type (google.cloud.channel_v1.types.CloudIdentityInfo.CustomerType):
            CustomerType indicates verification type
            needed for using services.
        primary_domain (str):
            Output only. The primary domain name.
        is_domain_verified (bool):
            Output only. Whether the domain is verified. This field is
            not returned for a Customer's cloud_identity_info resource.
            Partners can use the domains.get() method of the Workspace
            SDK's Directory API, or listen to the
            PRIMARY_DOMAIN_VERIFIED Pub/Sub event in to track domain
            verification of their resolve Workspace customers.
        alternate_email (str):
            The alternate email.
        phone_number (str):
            Phone number associated with the Cloud
            Identity.
        language_code (str):
            Language code.
        admin_console_uri (str):
            Output only. URI of Customer's Admin console
            dashboard.
        edu_data (google.cloud.channel_v1.types.EduData):
            Edu information about the customer.
    """

    class CustomerType(proto.Enum):
        r"""CustomerType of the customer"""
        CUSTOMER_TYPE_UNSPECIFIED = 0
        DOMAIN = 1
        TEAM = 2

    customer_type = proto.Field(proto.ENUM, number=1, enum=CustomerType,)
    primary_domain = proto.Field(proto.STRING, number=9,)
    is_domain_verified = proto.Field(proto.BOOL, number=4,)
    alternate_email = proto.Field(proto.STRING, number=6,)
    phone_number = proto.Field(proto.STRING, number=7,)
    language_code = proto.Field(proto.STRING, number=8,)
    admin_console_uri = proto.Field(proto.STRING, number=10,)
    edu_data = proto.Field(proto.MESSAGE, number=22, message="EduData",)


class Value(proto.Message):
    r"""Data type and value of a parameter.
    Attributes:
        int64_value (int):
            Represents an int64 value.
        string_value (str):
            Represents a string value.
        double_value (float):
            Represents a double value.
        proto_value (google.protobuf.any_pb2.Any):
            Represents an 'Any' proto value.
        bool_value (bool):
            Represents a boolean value.
    """

    int64_value = proto.Field(proto.INT64, number=1, oneof="kind",)
    string_value = proto.Field(proto.STRING, number=2, oneof="kind",)
    double_value = proto.Field(proto.DOUBLE, number=3, oneof="kind",)
    proto_value = proto.Field(
        proto.MESSAGE, number=4, oneof="kind", message=any_pb2.Any,
    )
    bool_value = proto.Field(proto.BOOL, number=5, oneof="kind",)


class AdminUser(proto.Message):
    r"""Information needed to create an Admin User for Google
    Workspace.

    Attributes:
        email (str):
            Primary email of the admin user.
        given_name (str):
            Given name of the admin user.
        family_name (str):
            Family name of the admin user.
    """

    email = proto.Field(proto.STRING, number=1,)
    given_name = proto.Field(proto.STRING, number=2,)
    family_name = proto.Field(proto.STRING, number=3,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.channel_v1.types import common
from google.protobuf import timestamp_pb2  # type: ignore
from google.type import postal_address_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.channel.v1", manifest={"Customer", "ContactInfo",},
)


class Customer(proto.Message):
    r"""Entity representing a customer of a reseller or distributor.
    Attributes:
        name (str):
            Output only. Resource name of the customer. Format:
            accounts/{account_id}/customers/{customer_id}
        org_display_name (str):
            Required. Name of the organization that the
            customer entity represents.
        org_postal_address (google.type.postal_address_pb2.PostalAddress):
            Required. The organization address for the
            customer. To enforce US laws and embargoes, we
            require a region and zip code. You must provide
            valid addresses for every customer. To set the
            customer's language, use the Customer-level
            language code.
        primary_contact_info (google.cloud.channel_v1.types.ContactInfo):
            Primary contact info.
        alternate_email (str):
            Secondary contact email. You need to provide
            an alternate email to create different domains
            if a primary contact email already exists. Users
            will receive a notification with credentials
            when you create an admin.google.com account.
            Secondary emails are also recovery email
            addresses.
        domain (str):
            Required. The customer's primary domain. Must
            match the primary contact email's domain.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Time when the customer was
            created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Time when the customer was
            updated.
        cloud_identity_id (str):
            Output only. The customer's Cloud Identity ID
            if the customer has a Cloud Identity resource.
        language_code (str):
            Optional. The BCP-47 language code, such as "en-US" or
            "sr-Latn". For more information, see
            https://www.unicode.org/reports/tr35/#Unicode_locale_identifier.
        cloud_identity_info (google.cloud.channel_v1.types.CloudIdentityInfo):
            Output only. Cloud Identity information for
            the customer. Populated only if a Cloud Identity
            account exists for this customer.
        channel_partner_id (str):
            Cloud Identity ID of the customer's channel
            partner. Populated only if a channel partner
            exists for this customer.
    """

    name = proto.Field(proto.STRING, number=1,)
    org_display_name = proto.Field(proto.STRING, number=2,)
    org_postal_address = proto.Field(
        proto.MESSAGE, number=3, message=postal_address_pb2.PostalAddress,
    )
    primary_contact_info = proto.Field(proto.MESSAGE, number=4, message="ContactInfo",)
    alternate_email = proto.Field(proto.STRING, number=5,)
    domain = proto.Field(proto.STRING, number=6,)
    create_time = proto.Field(proto.MESSAGE, number=7, message=timestamp_pb2.Timestamp,)
    update_time = proto.Field(proto.MESSAGE, number=8, message=timestamp_pb2.Timestamp,)
    cloud_identity_id = proto.Field(proto.STRING, number=9,)
    language_code = proto.Field(proto.STRING, number=10,)
    cloud_identity_info = proto.Field(
        proto.MESSAGE, number=12, message=common.CloudIdentityInfo,
    )
    channel_partner_id = proto.Field(proto.STRING, number=13,)


class ContactInfo(proto.Message):
    r"""Contact information for a customer account.
    Attributes:
        first_name (str):
            The customer account contact's first name.
        last_name (str):
            The customer account contact's last name.
        display_name (str):
            Output only. The customer account contact's
            display name, formatted as a combination of the
            customer's first and last name.
        email (str):
            The customer account's contact email.
            Required for entitlements that create
            admin.google.com accounts, and serves as the
            customer's username for those accounts.
        title (str):
            Optional. The customer account contact's job
            title.
        phone (str):
            The customer account's contact phone number.
    """

    first_name = proto.Field(proto.STRING, number=1,)
    last_name = proto.Field(proto.STRING, number=2,)
    display_name = proto.Field(proto.STRING, number=4,)
    email = proto.Field(proto.STRING, number=5,)
    title = proto.Field(proto.STRING, number=6,)
    phone = proto.Field(proto.STRING, number=7,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.channel.v1",
    manifest={"CustomerEvent", "EntitlementEvent", "SubscriberEvent",},
)


class CustomerEvent(proto.Message):
    r"""Represents Pub/Sub message content describing customer
    update.

    Attributes:
        customer (str):
            Resource name of the customer. Format:
            accounts/{account_id}/customers/{customer_id}
        event_type (google.cloud.channel_v1.types.CustomerEvent.Type):
            Type of event which happened on the customer.
    """

    class Type(proto.Enum):
        r"""Type of customer event."""
        TYPE_UNSPECIFIED = 0
        PRIMARY_DOMAIN_CHANGED = 1
        PRIMARY_DOMAIN_VERIFIED = 2

    customer = proto.Field(proto.STRING, number=1,)
    event_type = proto.Field(proto.ENUM, number=2, enum=Type,)


class EntitlementEvent(proto.Message):
    r"""Represents Pub/Sub message content describing entitlement
    update.

    Attributes:
        entitlement (str):
            Resource name of an entitlement of the form:
            accounts/{account_id}/customers/{customer_id}/entitlements/{entitlement_id}
        event_type (google.cloud.channel_v1.types.EntitlementEvent.Type):
            Type of event which happened on the
            entitlement.
    """

    class Type(proto.Enum):
        r"""Type of entitlement event."""
        TYPE_UNSPECIFIED = 0
        CREATED = 1
        PRICE_PLAN_SWITCHED = 3
        COMMITMENT_CHANGED = 4
        RENEWED = 5
        SUSPENDED = 6
        ACTIVATED = 7
        CANCELLED = 8
        SKU_CHANGED = 9
        RENEWAL_SETTING_CHANGED = 10
        PAID_SERVICE_STARTED = 11
        LICENSE_ASSIGNMENT_CHANGED = 12
        LICENSE_CAP_CHANGED = 13

    entitlement = proto.Field(proto.STRING, number=1,)
    event_type = proto.Field(proto.ENUM, number=2, enum=Type,)


class SubscriberEvent(proto.Message):
    r"""Represents information which resellers will get as part of
    notification from Cloud Pub/Sub.

    Attributes:
        customer_event (google.cloud.channel_v1.types.CustomerEvent):
            Customer event send as part of Pub/Sub event
            to partners.
        entitlement_event (google.cloud.channel_v1.types.EntitlementEvent):
            Entitlement event send as part of Pub/Sub
            event to partners.
    """

    customer_event = proto.Field(
        proto.MESSAGE, number=1, oneof="event", message="CustomerEvent",
    )
    entitlement_event = proto.Field(
        proto.MESSAGE, number=2, oneof="event", message="EntitlementEvent",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from .services.cloud_channel_service import CloudChannelServiceClient
from .services.cloud_channel_service import CloudChannelServiceAsyncClient

from .types.channel_partner_links import ChannelPartnerLink
from .types.channel_partner_links import ChannelPartnerLinkState
from .types.channel_partner_links import ChannelPartnerLinkView
from .types.common import AdminUser
from .types.common import CloudIdentityInfo
from .types.common import EduData
from .types.common import Value
from .types.customers import ContactInfo
from .types.customers import Customer
from .types.entitlements import AssociationInfo
from .types.entitlements import CommitmentSettings
from .types.entitlements import Entitlement
from .types.entitlements import Parameter
from .types.entitlements import ProvisionedService
from .types.entitlements import RenewalSettings
from .types.entitlements import TransferableSku
from .types.entitlements import TransferEligibility
from .types.entitlements import TrialSettings
from .types.offers import Constraints
from .types.offers import CustomerConstraints
from .types.offers import Offer
from .types.offers import ParameterDefinition
from .types.offers import Period
from .types.offers import Plan
from .types.offers import Price
from .types.offers import PriceByResource
from .types.offers import PricePhase
from .types.offers import PriceTier
from .types.offers import PaymentPlan
from .types.offers import PaymentType
from .types.offers import PeriodType
from .types.offers import PromotionalOrderType
from .types.offers import ResourceType
from .types.operations import OperationMetadata
from .types.products import MarketingInfo
from .types.products import Media
from .types.products import Product
from .types.products import Sku
from .types.products import MediaType
from .types.service import ActivateEntitlementRequest
from .types.service import CancelEntitlementRequest
from .types.service import ChangeOfferRequest
from .types.service import ChangeParametersRequest
from .types.service import ChangeRenewalSettingsRequest
from .types.service import CheckCloudIdentityAccountsExistRequest
from .types.service import CheckCloudIdentityAccountsExistResponse
from .types.service import CloudIdentityCustomerAccount
from .types.service import CreateChannelPartnerLinkRequest
from .types.service import CreateCustomerRequest
from .types.service import CreateEntitlementRequest
from .types.service import DeleteCustomerRequest
from .types.service import GetChannelPartnerLinkRequest
from .types.service import GetCustomerRequest
from .types.service import GetEntitlementRequest
from .types.service import ListChannelPartnerLinksRequest
from .types.service import ListChannelPartnerLinksResponse
from .types.service import ListCustomersRequest
from .types.service import ListCustomersResponse
from .types.service import ListEntitlementsRequest
from .types.service import ListEntitlementsResponse
from .types.service import ListOffersRequest
from .types.service import ListOffersResponse
from .types.service import ListProductsRequest
from .types.service import ListProductsResponse
from .types.service import ListPurchasableOffersRequest
from .types.service import ListPurchasableOffersResponse
from .types.service import ListPurchasableSkusRequest
from .types.service import ListPurchasableSkusResponse
from .types.service import ListSkusRequest
from .types.service import ListSkusResponse
from .types.service import ListSubscribersRequest
from .types.service import ListSubscribersResponse
from .types.service import ListTransferableOffersRequest
from .types.service import ListTransferableOffersResponse
from .types.service import ListTransferableSkusRequest
from .types.service import ListTransferableSkusResponse
from .types.service import LookupOfferRequest
from .types.service import ProvisionCloudIdentityRequest
from .types.service import PurchasableOffer
from .types.service import PurchasableSku
from .types.service import RegisterSubscriberRequest
from .types.service import RegisterSubscriberResponse
from .types.service import StartPaidServiceRequest
from .types.service import SuspendEntitlementRequest
from .types.service import TransferableOffer
from .types.service import TransferEntitlementsRequest
from .types.service import TransferEntitlementsResponse
from .types.service import TransferEntitlementsToGoogleRequest
from .types.service import UnregisterSubscriberRequest
from .types.service import UnregisterSubscriberResponse
from .types.service import UpdateChannelPartnerLinkRequest
from .types.service import UpdateCustomerRequest
from .types.subscriber_event import CustomerEvent
from .types.subscriber_event import EntitlementEvent
from .types.subscriber_event import SubscriberEvent

__all__ = (
    "CloudChannelServiceAsyncClient",
    "ActivateEntitlementRequest",
    "AdminUser",
    "AssociationInfo",
    "CancelEntitlementRequest",
    "ChangeOfferRequest",
    "ChangeParametersRequest",
    "ChangeRenewalSettingsRequest",
    "ChannelPartnerLink",
    "ChannelPartnerLinkState",
    "ChannelPartnerLinkView",
    "CheckCloudIdentityAccountsExistRequest",
    "CheckCloudIdentityAccountsExistResponse",
    "CloudChannelServiceClient",
    "CloudIdentityCustomerAccount",
    "CloudIdentityInfo",
    "CommitmentSettings",
    "Constraints",
    "ContactInfo",
    "CreateChannelPartnerLinkRequest",
    "CreateCustomerRequest",
    "CreateEntitlementRequest",
    "Customer",
    "CustomerConstraints",
    "CustomerEvent",
    "DeleteCustomerRequest",
    "EduData",
    "Entitlement",
    "EntitlementEvent",
    "GetChannelPartnerLinkRequest",
    "GetCustomerRequest",
    "GetEntitlementRequest",
    "ListChannelPartnerLinksRequest",
    "ListChannelPartnerLinksResponse",
    "ListCustomersRequest",
    "ListCustomersResponse",
    "ListEntitlementsRequest",
    "ListEntitlementsResponse",
    "ListOffersRequest",
    "ListOffersResponse",
    "ListProductsRequest",
    "ListProductsResponse",
    "ListPurchasableOffersRequest",
    "ListPurchasableOffersResponse",
    "ListPurchasableSkusRequest",
    "ListPurchasableSkusResponse",
    "ListSkusRequest",
    "ListSkusResponse",
    "ListSubscribersRequest",
    "ListSubscribersResponse",
    "ListTransferableOffersRequest",
    "ListTransferableOffersResponse",
    "ListTransferableSkusRequest",
    "ListTransferableSkusResponse",
    "LookupOfferRequest",
    "MarketingInfo",
    "Media",
    "MediaType",
    "Offer",
    "OperationMetadata",
    "Parameter",
    "ParameterDefinition",
    "PaymentPlan",
    "PaymentType",
    "Period",
    "PeriodType",
    "Plan",
    "Price",
    "PriceByResource",
    "PricePhase",
    "PriceTier",
    "Product",
    "PromotionalOrderType",
    "ProvisionCloudIdentityRequest",
    "ProvisionedService",
    "PurchasableOffer",
    "PurchasableSku",
    "RegisterSubscriberRequest",
    "RegisterSubscriberResponse",
    "RenewalSettings",
    "ResourceType",
    "Sku",
    "StartPaidServiceRequest",
    "SubscriberEvent",
    "SuspendEntitlementRequest",
    "TransferEligibility",
    "TransferEntitlementsRequest",
    "TransferEntitlementsResponse",
    "TransferEntitlementsToGoogleRequest",
    "TransferableOffer",
    "TransferableSku",
    "TrialSettings",
    "UnregisterSubscriberRequest",
    "UnregisterSubscriberResponse",
    "UpdateChannelPartnerLinkRequest",
    "UpdateCustomerRequest",
    "Value",
)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import Dict, Sequence, Tuple, Type, Union
import pkg_resources

import google.api_core.client_options as ClientOptions  # type: ignore
from google.api_core import exceptions as core_exceptions  # type: ignore
from google.api_core import gapic_v1  # type: ignore
from google.api_core import retry as retries  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.api_core import operation  # type: ignore
from google.api_core import operation_async  # type: ignore
from google.cloud.channel_v1.services.cloud_channel_service import pagers
from google.cloud.channel_v1.types import channel_partner_links
from google.cloud.channel_v1.types import common
from google.cloud.channel_v1.types import customers
from google.cloud.channel_v1.types import entitlements
from google.cloud.channel_v1.types import offers
from google.cloud.channel_v1.types import operations
from google.cloud.channel_v1.types import products
from google.cloud.channel_v1.types import service
from google.protobuf import empty_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from google.type import postal_address_pb2  # type: ignore
from .transports.base import CloudChannelServiceTransport, DEFAULT_CLIENT_INFO
from .transports.grpc_asyncio import CloudChannelServiceGrpcAsyncIOTransport
from .client import CloudChannelServiceClient


class CloudChannelServiceAsyncClient:
    """CloudChannelService lets Google cloud resellers and distributors
    manage their customers, channel partners, entitlements, and reports.

    Using this service:

    1. Resellers and distributors can manage a customer entity.
    2. Distributors can register an authorized reseller in their channel
       and provide them with delegated admin access.
    3. Resellers and distributors can manage customer entitlements.

    CloudChannelService exposes the following resources:

    -  [Customer][google.cloud.channel.v1.Customer]s: An entity—usually
       an enterprise—managed by a reseller or distributor.

    -  [Entitlement][google.cloud.channel.v1.Entitlement]s: An entity
       that provides a customer with the means to use a service.
       Entitlements are created or updated as a result of a successful
       fulfillment.

    -  [ChannelPartnerLink][google.cloud.channel.v1.ChannelPartnerLink]s:
       An entity that identifies links between distributors and their
       indirect resellers in a channel.
    """

    _client: CloudChannelServiceClient

    DEFAULT_ENDPOINT = CloudChannelServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = CloudChannelServiceClient.DEFAULT_MTLS_ENDPOINT

    customer_path = staticmethod(CloudChannelServiceClient.customer_path)
    parse_customer_path = staticmethod(CloudChannelServiceClient.parse_customer_path)
    entitlement_path = staticmethod(CloudChannelServiceClient.entitlement_path)
    parse_entitlement_path = staticmethod(
        CloudChannelServiceClient.parse_entitlement_path
    )
    offer_path = staticmethod(CloudChannelServiceClient.offer_path)
    parse_offer_path = staticmethod(CloudChannelServiceClient.parse_offer_path)
    product_path = staticmethod(CloudChannelServiceClient.product_path)
    parse_product_path = staticmethod(CloudChannelServiceClient.parse_product_path)
    sku_path = staticmethod(CloudChannelServiceClient.sku_path)
    parse_sku_path = staticmethod(CloudChannelServiceClient.parse_sku_path)
    common_billing_account_path = staticmethod(
        CloudChannelServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        CloudChannelServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(CloudChannelServiceClient.common_folder_path)
    parse_common_folder_path = staticmethod(
        CloudChannelServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        CloudChannelServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        CloudChannelServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(CloudChannelServiceClient.common_project_path)
    parse_common_project_path = staticmethod(
        CloudChannelServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(CloudChannelServiceClient.common_location_path)
    parse_common_location_path = staticmethod(
        CloudChannelServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            CloudChannelServiceAsyncClient: The constructed client.
        """
        return CloudChannelServiceClient.from_service_account_info.__func__(CloudChannelServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            CloudChannelServiceAsyncClient: The constructed client.
        """
        return CloudChannelServiceClient.from_service_account_file.__func__(CloudChannelServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @property
    def transport(self) -> CloudChannelServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            CloudChannelServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(CloudChannelServiceClient).get_transport_class,
        type(CloudChannelServiceClient),
    )

    def __init__(
        self,
        *,
        credentials: ga_credentials.Credentials = None,
        transport: Union[str, CloudChannelServiceTransport] = "grpc_asyncio",
        client_options: ClientOptions = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the cloud channel service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.CloudChannelServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = CloudChannelServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def list_customers(
        self,
        request: service.ListCustomersRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListCustomersAsyncPager:
        r"""List [Customer][google.cloud.channel.v1.Customer]s.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request is
           different from the reseller account in the API request.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.

        Return value: List of
        [Customer][google.cloud.channel.v1.Customer]s, or an empty list
        if there are no customers.

        Args:
            request (:class:`google.cloud.channel_v1.types.ListCustomersRequest`):
                The request object. Request message for
                [CloudChannelService.ListCustomers][google.cloud.channel.v1.CloudChannelService.ListCustomers]
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_service.pagers.ListCustomersAsyncPager:
                Response message for
                [CloudChannelService.ListCustomers][google.cloud.channel.v1.CloudChannelService.ListCustomers].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        request = service.ListCustomersRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_customers,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListCustomersAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_customer(
        self,
        request: service.GetCustomerRequest = None,
        *,
        name: str = None,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> customers.Customer:
        r"""Returns the requested
        [Customer][google.cloud.channel.v1.Customer] resource.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request is
           different from the reseller account in the API request.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: The customer resource doesn't exist. Usually the
           result of an invalid name parameter.

        Return value: The [Customer][google.cloud.channel.v1.Customer]
        resource.

        Args:
            request (:class:`google.cloud.channel_v1.types.GetCustomerRequest`):
                The request object. Request message for
                [CloudChannelService.GetCustomer][google.cloud.channel.v1.CloudChannelService.GetCustomer].
            name (:class:`str`):
                Required. The resource name of the customer to retrieve.
                Name uses the format:
                accounts/{account_id}/customers/{customer_id}

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.types.Customer:
                Entity representing a customer of a
                reseller or distributor.

        """
        # Create or coerce a protobuf request object.
        # Sanity check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = service.GetCustomerRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_customer,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def check_cloud_identity_accounts_exist(
        self,
        request: service.CheckCloudIdentityAccountsExistRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> service.CheckCloudIdentityAccountsExistResponse:
        r"""Confirms the existence of Cloud Identity accounts based on the
        domain and if the Cloud Identity accounts are owned by the
        reseller.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request is
           different from the reseller account in the API request.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  INVALID_VALUE: Invalid domain value in the request.

        Return value: A list of
        [CloudIdentityCustomerAccount][google.cloud.channel.v1.CloudIdentityCustomerAccount]
        resources for the domain (may be empty)

        Note: in the v1alpha1 version of the API, a NOT_FOUND error
        returns if no
        [CloudIdentityCustomerAccount][google.cloud.channel.v1.CloudIdentityCustomerAccount]
        resources match the domain.

        Args:
            request (:class:`google.cloud.channel_v1.types.CheckCloudIdentityAccountsExistRequest`):
                The request object. Request message for
                [CloudChannelService.CheckCloudIdentityAccountsExist][google.cloud.channel.v1.CloudChannelService.CheckCloudIdentityAccountsExist].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.types.CheckCloudIdentityAccountsExistResponse:
                Response message for
                   [CloudChannelService.CheckCloudIdentityAccountsExist][google.cloud.channel.v1.CloudChannelService.CheckCloudIdentityAccountsExist].

        """
        # Create or coerce a protobuf request object.
        request = service.CheckCloudIdentityAccountsExistRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.check_cloud_identity_accounts_exist,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def create_customer(
        self,
        request: service.CreateCustomerRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> customers.Customer:
        r"""Creates a new [Customer][google.cloud.channel.v1.Customer]
        resource under the reseller or distributor account.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request is
           different from the reseller account in the API request.
        -  INVALID_ARGUMENT:

           -  Required request parameters are missing or invalid.
           -  Domain field value doesn't match the primary email domain.

        Return value: The newly created
        [Customer][google.cloud.channel.v1.Customer] resource.

        Args:
            request (:class:`google.cloud.channel_v1.types.CreateCustomerRequest`):
                The request object. Request message for
                [CloudChannelService.CreateCustomer][google.cloud.channel.v1.CloudChannelService.CreateCustomer]
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.types.Customer:
                Entity representing a customer of a
                reseller or distributor.

        """
        # Create or coerce a protobuf request object.
        request = service.CreateCustomerRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_customer,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def update_customer(
        self,
        request: service.UpdateCustomerRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> customers.Customer:
        r"""Updates an existing [Customer][google.cloud.channel.v1.Customer]
        resource for the reseller or distributor.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request is
           different from the reseller account in the API request.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: No [Customer][google.cloud.channel.v1.Customer]
           resource found for the name in the request.

        Return value: The updated
        [Customer][google.cloud.channel.v1.Customer] resource.

        Args:
            request (:class:`google.cloud.channel_v1.types.UpdateCustomerRequest`):
                The request object. Request message for
                [CloudChannelService.UpdateCustomer][google.cloud.channel.v1.CloudChannelService.UpdateCustomer].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.types.Customer:
                Entity representing a customer of a
                reseller or distributor.

        """
        # Create or coerce a protobuf request object.
        request = service.UpdateCustomerRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_customer,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("customer.name", request.customer.name),)
            ),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def delete_customer(
        self,
        request: service.DeleteCustomerRequest = None,
        *,
        name: str = None,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Deletes the given [Customer][google.cloud.channel.v1.Customer]
        permanently.

        Possible error codes:

        -  PERMISSION_DENIED: The account making the request does not
           own this customer.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  FAILED_PRECONDITION: The customer has existing entitlements.
        -  NOT_FOUND: No [Customer][google.cloud.channel.v1.Customer]
           resource found for the name in the request.

        Args:
            request (:class:`google.cloud.channel_v1.types.DeleteCustomerRequest`):
                The request object. Request message for
                [CloudChannelService.DeleteCustomer][google.cloud.channel.v1.CloudChannelService.DeleteCustomer].
            name (:class:`str`):
                Required. The resource name of the
                customer to delete.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        # Create or coerce a protobuf request object.
        # Sanity check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = service.DeleteCustomerRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.delete_customer,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        await rpc(
            request, retry=retry, timeout=timeout, metadata=metadata,
        )

    async def provision_cloud_identity(
        self,
        request: service.ProvisionCloudIdentityRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Creates a Cloud Identity for the given customer using the
        customer's information, or the information provided here.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: The customer was not found.
        -  ALREADY_EXISTS: The customer's primary email already exists.
           Retry after changing the customer's primary contact email.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The ID of a long-running operation.

        To get the results of the operation, call the GetOperation
        method of CloudChannelOperationsService. The Operation metadata
        contains an instance of
        [OperationMetadata][google.cloud.channel.v1.OperationMetadata].

        Args:
            request (:class:`google.cloud.channel_v1.types.ProvisionCloudIdentityRequest`):
                The request object. Request message for
                [CloudChannelService.ProvisionCloudIdentity][google.cloud.channel.v1.CloudChannelService.ProvisionCloudIdentity]
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be
                :class:`google.cloud.channel_v1.types.Customer` Entity
                representing a customer of a reseller or distributor.

        """
        # Create or coerce a protobuf request object.
        request = service.ProvisionCloudIdentityRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.provision_cloud_identity,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("customer", request.customer),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            customers.Customer,
            metadata_type=operations.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def list_entitlements(
        self,
        request: service.ListEntitlementsRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListEntitlementsAsyncPager:
        r"""Lists [Entitlement][google.cloud.channel.v1.Entitlement]s
        belonging to a customer.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.

        Return value: A list of the customer's
        [Entitlement][google.cloud.channel.v1.Entitlement]s.

        Args:
            request (:class:`google.cloud.channel_v1.types.ListEntitlementsRequest`):
                The request object. Request message for
                [CloudChannelService.ListEntitlements][google.cloud.channel.v1.CloudChannelService.ListEntitlements]
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_service.pagers.ListEntitlementsAsyncPager:
                Response message for
                [CloudChannelService.ListEntitlements][google.cloud.channel.v1.CloudChannelService.ListEntitlements].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        request = service.ListEntitlementsRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_entitlements,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListEntitlementsAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_transferable_skus(
        self,
        request: service.ListTransferableSkusRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListTransferableSkusAsyncPager:
        r"""List [TransferableSku][google.cloud.channel.v1.TransferableSku]s
        of a customer based on the Cloud Identity ID or Customer Name in
        the request.

        Use this method to list the entitlements information of an
        unowned customer. You should provide the customer's Cloud
        Identity ID or Customer Name.

        Possible error codes:

        -  PERMISSION_DENIED:

           -  The customer doesn't belong to the reseller and has no
              auth token.
           -  The supplied auth token is invalid.
           -  The reseller account making the request is different from
              the reseller account in the query.

        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.

        Return value: A list of the customer's
        [TransferableSku][google.cloud.channel.v1.TransferableSku].

        Args:
            request (:class:`google.cloud.channel_v1.types.ListTransferableSkusRequest`):
                The request object. Request message for
                [CloudChannelService.ListTransferableSkus][google.cloud.channel.v1.CloudChannelService.ListTransferableSkus]
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_service.pagers.ListTransferableSkusAsyncPager:
                Response message for
                [CloudChannelService.ListTransferableSkus][google.cloud.channel.v1.CloudChannelService.ListTransferableSkus].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        request = service.ListTransferableSkusRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_transferable_skus,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListTransferableSkusAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_transferable_offers(
        self,
        request: service.ListTransferableOffersRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListTransferableOffersAsyncPager:
        r"""List
        [TransferableOffer][google.cloud.channel.v1.TransferableOffer]s
        of a customer based on Cloud Identity ID or Customer Name in the
        request.

        Use this method when a reseller gets the entitlement information
        of an unowned customer. The reseller should provide the
        customer's Cloud Identity ID or Customer Name.

        Possible error codes:

        -  PERMISSION_DENIED:

           -  The customer doesn't belong to the reseller and has no
              auth token.
           -  The supplied auth token is invalid.
           -  The reseller account making the request is different from
              the reseller account in the query.

        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.

        Return value: List of
        [TransferableOffer][google.cloud.channel.v1.TransferableOffer]
        for the given customer and SKU.

        Args:
            request (:class:`google.cloud.channel_v1.types.ListTransferableOffersRequest`):
                The request object. Request message for
                [CloudChannelService.ListTransferableOffers][google.cloud.channel.v1.CloudChannelService.ListTransferableOffers]
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_service.pagers.ListTransferableOffersAsyncPager:
                Response message for
                [CloudChannelService.ListTransferableOffers][google.cloud.channel.v1.CloudChannelService.ListTransferableOffers].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        request = service.ListTransferableOffersRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_transferable_offers,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListTransferableOffersAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_entitlement(
        self,
        request: service.GetEntitlementRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> entitlements.Entitlement:
        r"""Returns the requested
        [Entitlement][google.cloud.channel.v1.Entitlement] resource.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: The customer entitlement was not found.

        Return value: The requested
        [Entitlement][google.cloud.channel.v1.Entitlement] resource.

        Args:
            request (:class:`google.cloud.channel_v1.types.GetEntitlementRequest`):
                The request object. Request message for
                [CloudChannelService.GetEntitlement][google.cloud.channel.v1.CloudChannelService.GetEntitlement].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.types.Entitlement:
                An entitlement is a representation of
                a customer's ability to use a service.

        """
        # Create or coerce a protobuf request object.
        request = service.GetEntitlementRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_entitlement,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def create_entitlement(
        self,
        request: service.CreateEntitlementRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Creates an entitlement for a customer.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT:

           -  Required request parameters are missing or invalid.
           -  There is already a customer entitlement for a SKU from the
              same product family.

        -  INVALID_VALUE: Make sure the OfferId is valid. If it is,
           contact Google Channel support for further troubleshooting.
        -  NOT_FOUND: The customer or offer resource was not found.
        -  ALREADY_EXISTS:

           -  The SKU was already purchased for the customer.
           -  The customer's primary email already exists. Retry after
              changing the customer's primary contact email.

        -  CONDITION_NOT_MET or FAILED_PRECONDITION:

           -  The domain required for purchasing a SKU has not been
              verified.
           -  A pre-requisite SKU required to purchase an Add-On SKU is
              missing. For example, Google Workspace Business Starter is
              required to purchase Vault or Drive.
           -  (Developer accounts only) Reseller and resold domain must
              meet the following naming requirements:

              -  Domain names must start with goog-test.
              -  Domain names must include the reseller domain.

        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The ID of a long-running operation.

        To get the results of the operation, call the GetOperation
        method of CloudChannelOperationsService. The Operation metadata
        will contain an instance of
        [OperationMetadata][google.cloud.channel.v1.OperationMetadata].

        Args:
            request (:class:`google.cloud.channel_v1.types.CreateEntitlementRequest`):
                The request object. Request message for
                [CloudChannelService.CreateEntitlement][google.cloud.channel.v1.CloudChannelService.CreateEntitlement]
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be
                :class:`google.cloud.channel_v1.types.Entitlement` An
                entitlement is a representation of a customer's ability
                to use a service.

        """
        # Create or coerce a protobuf request object.
        request = service.CreateEntitlementRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_entitlement,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            entitlements.Entitlement,
            metadata_type=operations.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def change_parameters(
        self,
        request: service.ChangeParametersRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Change parameters of the entitlement.

        An entitlement update is a long-running operation and it updates
        the entitlement as a result of fulfillment.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid. For example, the number of seats being changed is
           greater than the allowed number of max seats, or decreasing
           seats for a commitment based plan.
        -  NOT_FOUND: Entitlement resource not found.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The ID of a long-running operation.

        To get the results of the operation, call the GetOperation
        method of CloudChannelOperationsService. The Operation metadata
        will contain an instance of
        [OperationMetadata][google.cloud.channel.v1.OperationMetadata].

        Args:
            request (:class:`google.cloud.channel_v1.types.ChangeParametersRequest`):
                The request object. Request message for
                [CloudChannelService.ChangeParametersRequest][].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be
                :class:`google.cloud.channel_v1.types.Entitlement` An
                entitlement is a representation of a customer's ability
                to use a service.

        """
        # Create or coerce a protobuf request object.
        request = service.ChangeParametersRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.change_parameters,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            entitlements.Entitlement,
            metadata_type=operations.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def change_renewal_settings(
        self,
        request: service.ChangeRenewalSettingsRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Updates the renewal settings for an existing customer
        entitlement.

        An entitlement update is a long-running operation and it updates
        the entitlement as a result of fulfillment.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: Entitlement resource not found.
        -  NOT_COMMITMENT_PLAN: Renewal Settings are only applicable for
           a commitment plan. Can't enable or disable renewals for
           non-commitment plans.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The ID of a long-running operation.

        To get the results of the operation, call the GetOperation
        method of CloudChannelOperationsService. The Operation metadata
        will contain an instance of
        [OperationMetadata][google.cloud.channel.v1.OperationMetadata].

        Args:
            request (:class:`google.cloud.channel_v1.types.ChangeRenewalSettingsRequest`):
                The request object. Request message for
                [CloudChannelService.ChangeRenewalSettings][google.cloud.channel.v1.CloudChannelService.ChangeRenewalSettings].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be
                :class:`google.cloud.channel_v1.types.Entitlement` An
                entitlement is a representation of a customer's ability
                to use a service.

        """
        # Create or coerce a protobuf request object.
        request = service.ChangeRenewalSettingsRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.change_renewal_settings,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            entitlements.Entitlement,
            metadata_type=operations.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def change_offer(
        self,
        request: service.ChangeOfferRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Updates the Offer for an existing customer entitlement.

        An entitlement update is a long-running operation and it updates
        the entitlement as a result of fulfillment.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: Offer or Entitlement resource not found.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The ID of a long-running operation.

        To get the results of the operation, call the GetOperation
        method of CloudChannelOperationsService. The Operation metadata
        will contain an instance of
        [OperationMetadata][google.cloud.channel.v1.OperationMetadata].

        Args:
            request (:class:`google.cloud.channel_v1.types.ChangeOfferRequest`):
                The request object. Request message for
                [CloudChannelService.ChangeOffer][google.cloud.channel.v1.CloudChannelService.ChangeOffer].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be
                :class:`google.cloud.channel_v1.types.Entitlement` An
                entitlement is a representation of a customer's ability
                to use a service.

        """
        # Create or coerce a protobuf request object.
        request = service.ChangeOfferRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.change_offer,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            entitlements.Entitlement,
            metadata_type=operations.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def start_paid_service(
        self,
        request: service.StartPaidServiceRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Starts paid service for a trial entitlement.

        Starts paid service for a trial entitlement immediately. This
        method is only applicable if a plan is set up for a trial
        entitlement but has some trial days remaining.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: Entitlement resource not found.
        -  FAILED_PRECONDITION/NOT_IN_TRIAL: This method only works for
           entitlement on trial plans.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The ID of a long-running operation.

        To get the results of the operation, call the GetOperation
        method of CloudChannelOperationsService. The Operation metadata
        will contain an instance of
        [OperationMetadata][google.cloud.channel.v1.OperationMetadata].

        Args:
            request (:class:`google.cloud.channel_v1.types.StartPaidServiceRequest`):
                The request object. Request message for
                [CloudChannelService.StartPaidService][google.cloud.channel.v1.CloudChannelService.StartPaidService].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be
                :class:`google.cloud.channel_v1.types.Entitlement` An
                entitlement is a representation of a customer's ability
                to use a service.

        """
        # Create or coerce a protobuf request object.
        request = service.StartPaidServiceRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.start_paid_service,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            entitlements.Entitlement,
            metadata_type=operations.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def suspend_entitlement(
        self,
        request: service.SuspendEntitlementRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Suspends a previously fulfilled entitlement.

        An entitlement suspension is a long-running operation.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: Entitlement resource not found.
        -  NOT_ACTIVE: Entitlement is not active.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The ID of a long-running operation.

        To get the results of the operation, call the GetOperation
        method of CloudChannelOperationsService. The Operation metadata
        will contain an instance of
        [OperationMetadata][google.cloud.channel.v1.OperationMetadata].

        Args:
            request (:class:`google.cloud.channel_v1.types.SuspendEntitlementRequest`):
                The request object. Request message for
                [CloudChannelService.SuspendEntitlement][google.cloud.channel.v1.CloudChannelService.SuspendEntitlement].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be
                :class:`google.cloud.channel_v1.types.Entitlement` An
                entitlement is a representation of a customer's ability
                to use a service.

        """
        # Create or coerce a protobuf request object.
        request = service.SuspendEntitlementRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.suspend_entitlement,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            entitlements.Entitlement,
            metadata_type=operations.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def cancel_entitlement(
        self,
        request: service.CancelEntitlementRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Cancels a previously fulfilled entitlement.

        An entitlement cancellation is a long-running operation.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request is
           different from the reseller account in the API request.
        -  FAILED_PRECONDITION: There are Google Cloud projects linked
           to the Google Cloud entitlement's Cloud Billing subaccount.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: Entitlement resource not found.
        -  DELETION_TYPE_NOT_ALLOWED: Cancel is only allowed for Google
           Workspace add-ons, or entitlements for Google Cloud's
           development platform.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The ID of a long-running operation.

        To get the results of the operation, call the GetOperation
        method of CloudChannelOperationsService. The response will
        contain google.protobuf.Empty on success. The Operation metadata
        will contain an instance of
        [OperationMetadata][google.cloud.channel.v1.OperationMetadata].

        Args:
            request (:class:`google.cloud.channel_v1.types.CancelEntitlementRequest`):
                The request object. Request message for
                [CloudChannelService.CancelEntitlement][google.cloud.channel.v1.CloudChannelService.CancelEntitlement].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

                   The JSON representation for Empty is empty JSON
                   object {}.

        """
        # Create or coerce a protobuf request object.
        request = service.CancelEntitlementRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.cancel_entitlement,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=operations.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def activate_entitlement(
        self,
        request: service.ActivateEntitlementRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Activates a previously suspended entitlement. Entitlements
        suspended for pending ToS acceptance can't be activated using
        this method.

        An entitlement activation is a long-running operation and it
        updates the state of the customer entitlement.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request is
           different from the reseller account in the API request.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: Entitlement resource not found.
        -  SUSPENSION_NOT_RESELLER_INITIATED: Can only activate
           reseller-initiated suspensions and entitlements that have
           accepted the TOS.
        -  NOT_SUSPENDED: Can only activate suspended entitlements not
           in an ACTIVE state.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The ID of a long-running operation.

        To get the results of the operation, call the GetOperation
        method of CloudChannelOperationsService. The Operation metadata
        will contain an instance of
        [OperationMetadata][google.cloud.channel.v1.OperationMetadata].

        Args:
            request (:class:`google.cloud.channel_v1.types.ActivateEntitlementRequest`):
                The request object. Request message for
                [CloudChannelService.ActivateEntitlement][google.cloud.channel.v1.CloudChannelService.ActivateEntitlement].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be
                :class:`google.cloud.channel_v1.types.Entitlement` An
                entitlement is a representation of a customer's ability
                to use a service.

        """
        # Create or coerce a protobuf request object.
        request = service.ActivateEntitlementRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.activate_entitlement,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            entitlements.Entitlement,
            metadata_type=operations.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def transfer_entitlements(
        self,
        request: service.TransferEntitlementsRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Transfers customer entitlements to new reseller.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: The customer or offer resource was not found.
        -  ALREADY_EXISTS: The SKU was already transferred for the
           customer.
        -  CONDITION_NOT_MET or FAILED_PRECONDITION:

           -  The SKU requires domain verification to transfer, but the
              domain is not verified.
           -  An Add-On SKU (example, Vault or Drive) is missing the
              pre-requisite SKU (example, G Suite Basic).
           -  (Developer accounts only) Reseller and resold domain must
              meet the following naming requirements:

              -  Domain names must start with goog-test.
              -  Domain names must include the reseller domain.

           -  Specify all transferring entitlements.

        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The ID of a long-running operation.

        To get the results of the operation, call the GetOperation
        method of CloudChannelOperationsService. The Operation metadata
        will contain an instance of
        [OperationMetadata][google.cloud.channel.v1.OperationMetadata].

        Args:
            request (:class:`google.cloud.channel_v1.types.TransferEntitlementsRequest`):
                The request object. Request message for
                [CloudChannelService.TransferEntitlements][google.cloud.channel.v1.CloudChannelService.TransferEntitlements].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.cloud.channel_v1.types.TransferEntitlementsResponse` Response message for [CloudChannelService.TransferEntitlements][google.cloud.channel.v1.CloudChannelService.TransferEntitlements].
                   This is put in the response field of
                   google.longrunning.Operation.

        """
        # Create or coerce a protobuf request object.
        request = service.TransferEntitlementsRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.transfer_entitlements,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            service.TransferEntitlementsResponse,
            metadata_type=operations.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def transfer_entitlements_to_google(
        self,
        request: service.TransferEntitlementsToGoogleRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation_async.AsyncOperation:
        r"""Transfers customer entitlements from their current reseller to
        Google.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: The customer or offer resource was not found.
        -  ALREADY_EXISTS: The SKU was already transferred for the
           customer.
        -  CONDITION_NOT_MET or FAILED_PRECONDITION:

           -  The SKU requires domain verification to transfer, but the
              domain is not verified.
           -  An Add-On SKU (example, Vault or Drive) is missing the
              pre-requisite SKU (example, G Suite Basic).
           -  (Developer accounts only) Reseller and resold domain must
              meet the following naming requirements:

              -  Domain names must start with goog-test.
              -  Domain names must include the reseller domain.

        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The ID of a long-running operation.

        To get the results of the operation, call the GetOperation
        method of CloudChannelOperationsService. The response will
        contain google.protobuf.Empty on success. The Operation metadata
        will contain an instance of
        [OperationMetadata][google.cloud.channel.v1.OperationMetadata].

        Args:
            request (:class:`google.cloud.channel_v1.types.TransferEntitlementsToGoogleRequest`):
                The request object. Request message for
                [CloudChannelService.TransferEntitlementsToGoogle][google.cloud.channel.v1.CloudChannelService.TransferEntitlementsToGoogle].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation_async.AsyncOperation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

                   The JSON representation for Empty is empty JSON
                   object {}.

        """
        # Create or coerce a protobuf request object.
        request = service.TransferEntitlementsToGoogleRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.transfer_entitlements_to_google,
            default_timeout=60.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Wrap the response in an operation future.
        response = operation_async.from_gapic(
            response,
            self._client._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=operations.OperationMetadata,
        )

        # Done; return the response.
        return response

    async def list_channel_partner_links(
        self,
        request: service.ListChannelPartnerLinksRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListChannelPartnerLinksAsyncPager:
        r"""List
        [ChannelPartnerLink][google.cloud.channel.v1.ChannelPartnerLink]s
        belonging to a distributor. You must be a distributor to call
        this method.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request is
           different from the reseller account in the API request.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.

        Return value: The list of the distributor account's
        [ChannelPartnerLink][google.cloud.channel.v1.ChannelPartnerLink]
        resources.

        Args:
            request (:class:`google.cloud.channel_v1.types.ListChannelPartnerLinksRequest`):
                The request object. Request message for
                [CloudChannelService.ListChannelPartnerLinks][google.cloud.channel.v1.CloudChannelService.ListChannelPartnerLinks]
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_service.pagers.ListChannelPartnerLinksAsyncPager:
                Response message for
                [CloudChannelService.ListChannelPartnerLinks][google.cloud.channel.v1.CloudChannelService.ListChannelPartnerLinks].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        request = service.ListChannelPartnerLinksRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_channel_partner_links,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListChannelPartnerLinksAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_channel_partner_link(
        self,
        request: service.GetChannelPartnerLinkRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> channel_partner_links.ChannelPartnerLink:
        r"""Returns the requested
        [ChannelPartnerLink][google.cloud.channel.v1.ChannelPartnerLink]
        resource. You must be a distributor to call this method.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request is
           different from the reseller account in the API request.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: ChannelPartnerLink resource not found because of
           an invalid channel partner link name.

        Return value: The
        [ChannelPartnerLink][google.cloud.channel.v1.ChannelPartnerLink]
        resource.

        Args:
            request (:class:`google.cloud.channel_v1.types.GetChannelPartnerLinkRequest`):
                The request object. Request message for
                [CloudChannelService.GetChannelPartnerLink][google.cloud.channel.v1.CloudChannelService.GetChannelPartnerLink].
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.types.ChannelPartnerLink:
                Entity representing a link between
                distributors and their indirect
                resellers in an n-tier resale channel.

        """
        # Create or coerce a protobuf request object.
        request = service.GetChannelPartnerLinkRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_channel_partner_link,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def create_channel_partner_link(
        self,
        request: service.CreateChannelPartnerLinkRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> channel_partner_links.ChannelPartnerLink:
        r"""Initiates a channel partner link between a distributor and a
        reseller, or between resellers in an n-tier reseller channel.
        Invited partners need to follow the invite_link_uri provided in
        the response to accept. After accepting the invitation, a link
        is set up between the two parties. You must be a distributor to
        call this method.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request is
           different from the reseller account in the API request.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  ALREADY_EXISTS: The ChannelPartnerLink sent in the request
           already exists.
        -  NOT_FOUND: No Cloud Identity customer exists for provided
           domain.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The new
        [ChannelPartnerLink][google.cloud.channel.v1.ChannelPartnerLink]
        resource.

        Args:
            request (:class:`google.cloud.channel_v1.types.CreateChannelPartnerLinkRequest`):
                The request object. Request message for
                [CloudChannelService.CreateChannelPartnerLink][google.cloud.channel.v1.CloudChannelService.CreateChannelPartnerLink]
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.types.ChannelPartnerLink:
                Entity representing a link between
                distributors and their indirect
                resellers in an n-tier resale channel.

        """
        # Create or coerce a protobuf request object.
        request = service.CreateChannelPartnerLinkRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_channel_partner_link,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def update_channel_partner_link(
        self,
        request: service.UpdateChannelPartnerLinkRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> channel_partner_links.ChannelPartnerLink:
        r"""Updates a channel partner link. Distributors call this method to
        change a link's status. For example, to suspend a partner link.
        You must be a distributor to call this method.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request is
           different from the reseller account in the API request.
        -  INVALID_ARGUMENT:

           -  Required request parameters are missing or invalid.
           -  Link state cannot change from invited to active or
              suspended.
           -  Cannot send reseller_cloud_identity_id, invite_url, or
              name in update mask.

        -  NOT_FOUND: ChannelPartnerLink resource not found.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The updated
        [ChannelPartnerLink][google.cloud.channel.v1.ChannelPartnerLink]
        resource.

        Args:
            request (:class:`google.cloud.channel_v1.types.UpdateChannelPartnerLinkRequest`):
                The request object. Request message for
                [CloudChannelService.UpdateChannelPartnerLink][google.cloud.channel.v1.CloudChannelService.UpdateChannelPartnerLink]
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.types.ChannelPartnerLink:
                Entity representing a link between
                distributors and their indirect
                resellers in an n-tier resale channel.

        """
        # Create or coerce a protobuf request object.
        request = service.UpdateChannelPartnerLinkRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_channel_partner_link,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def lookup_offer(
        self,
        request: service.LookupOfferRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> offers.Offer:
        r"""Returns the requested [Offer][google.cloud.channel.v1.Offer]
        resource.

        Possible error codes:

        -  PERMISSION_DENIED: The entitlement doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: Entitlement or offer was not found.

        Return value: The [Offer][google.cloud.channel.v1.Offer]
        resource.

        Args:
            request (:class:`google.cloud.channel_v1.types.LookupOfferRequest`):
                The request object. Request message for LookupOffer.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.types.Offer:
                Represents an offer made to resellers for purchase.
                   An offer is associated with a
                   [Sku][google.cloud.channel.v1.Sku], has a plan for
                   payment, a price, and defines the constraints for
                   buying.

        """
        # Create or coerce a protobuf request object.
        request = service.LookupOfferRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.lookup_offer,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("entitlement", request.entitlement),)
            ),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def list_products(
        self,
        request: service.ListProductsRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListProductsAsyncPager:
        r"""Lists the Products the reseller is authorized to sell.

        Possible error codes:

        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.

        Args:
            request (:class:`google.cloud.channel_v1.types.ListProductsRequest`):
                The request object. Request message for ListProducts.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_service.pagers.ListProductsAsyncPager:
                Response message for ListProducts.
                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        request = service.ListProductsRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_products,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListProductsAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_skus(
        self,
        request: service.ListSkusRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListSkusAsyncPager:
        r"""Lists the SKUs for a product the reseller is authorized to sell.

        Possible error codes:

        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.

        Args:
            request (:class:`google.cloud.channel_v1.types.ListSkusRequest`):
                The request object. Request message for ListSkus.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_service.pagers.ListSkusAsyncPager:
                Response message for ListSkus.
                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        request = service.ListSkusRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_skus,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListSkusAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_offers(
        self,
        request: service.ListOffersRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListOffersAsyncPager:
        r"""Lists the Offers the reseller can sell.

        Possible error codes:

        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.

        Args:
            request (:class:`google.cloud.channel_v1.types.ListOffersRequest`):
                The request object. Request message for ListOffers.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_service.pagers.ListOffersAsyncPager:
                Response message for ListOffers.
                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        request = service.ListOffersRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_offers,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListOffersAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_purchasable_skus(
        self,
        request: service.ListPurchasableSkusRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListPurchasableSkusAsyncPager:
        r"""Lists the following:

        -  SKUs that you can purchase for a customer
        -  SKUs that you can upgrade or downgrade for an entitlement.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.

        Args:
            request (:class:`google.cloud.channel_v1.types.ListPurchasableSkusRequest`):
                The request object. Request message for
                ListPurchasableSkus.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_service.pagers.ListPurchasableSkusAsyncPager:
                Response message for
                ListPurchasableSkus.
                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        request = service.ListPurchasableSkusRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_purchasable_skus,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("customer", request.customer),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListPurchasableSkusAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_purchasable_offers(
        self,
        request: service.ListPurchasableOffersRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListPurchasableOffersAsyncPager:
        r"""Lists the following:

        -  Offers that you can purchase for a customer.
        -  Offers that you can change for an entitlement.

        Possible error codes:

        -  PERMISSION_DENIED: The customer doesn't belong to the
           reseller
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.

        Args:
            request (:class:`google.cloud.channel_v1.types.ListPurchasableOffersRequest`):
                The request object. Request message for
                ListPurchasableOffers.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_service.pagers.ListPurchasableOffersAsyncPager:
                Response message for
                ListPurchasableOffers.
                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        request = service.ListPurchasableOffersRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_purchasable_offers,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("customer", request.customer),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListPurchasableOffersAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def register_subscriber(
        self,
        request: service.RegisterSubscriberRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> service.RegisterSubscriberResponse:
        r"""Registers a service account with subscriber privileges on the
        Cloud Pub/Sub topic for this Channel Services account. After you
        create a subscriber, you get the events through
        [SubscriberEvent][google.cloud.channel.v1.SubscriberEvent]

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request
           and the provided reseller account are different, or the
           impersonated user is not a super admin.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The topic name with the registered service email
        address.

        Args:
            request (:class:`google.cloud.channel_v1.types.RegisterSubscriberRequest`):
                The request object. Request Message for
                RegisterSubscriber.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.types.RegisterSubscriberResponse:
                Response Message for
                RegisterSubscriber.

        """
        # Create or coerce a protobuf request object.
        request = service.RegisterSubscriberRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.register_subscriber,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("account", request.account),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def unregister_subscriber(
        self,
        request: service.UnregisterSubscriberRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> service.UnregisterSubscriberResponse:
        r"""Unregisters a service account with subscriber privileges on the
        Cloud Pub/Sub topic created for this Channel Services account.
        If there are no service accounts left with subscriber
        privileges, this deletes the topic. You can call ListSubscribers
        to check for these accounts.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request
           and the provided reseller account are different, or the
           impersonated user is not a super admin.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: The topic resource doesn't exist.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: The topic name that unregistered the service email
        address. Returns a success response if the service email address
        wasn't registered with the topic.

        Args:
            request (:class:`google.cloud.channel_v1.types.UnregisterSubscriberRequest`):
                The request object. Request Message for
                UnregisterSubscriber.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.types.UnregisterSubscriberResponse:
                Response Message for
                UnregisterSubscriber.

        """
        # Create or coerce a protobuf request object.
        request = service.UnregisterSubscriberRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.unregister_subscriber,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("account", request.account),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def list_subscribers(
        self,
        request: service.ListSubscribersRequest = None,
        *,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListSubscribersAsyncPager:
        r"""Lists service accounts with subscriber privileges on the Cloud
        Pub/Sub topic created for this Channel Services account.

        Possible error codes:

        -  PERMISSION_DENIED: The reseller account making the request
           and the provided reseller account are different, or the
           impersonated user is not a super admin.
        -  INVALID_ARGUMENT: Required request parameters are missing or
           invalid.
        -  NOT_FOUND: The topic resource doesn't exist.
        -  INTERNAL: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.
        -  UNKNOWN: Any non-user error related to a technical issue in
           the backend. Contact Cloud Channel support.

        Return value: A list of service email addresses.

        Args:
            request (:class:`google.cloud.channel_v1.types.ListSubscribersRequest`):
                The request object. Request Message for ListSubscribers.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.channel_v1.services.cloud_channel_service.pagers.ListSubscribersAsyncPager:
                Response Message for ListSubscribers.
                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        request = service.ListSubscribersRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_subscribers,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("account", request.account),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListSubscribersAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response


try:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution("google-cloud-channel",).version,
    )
except pkg_resources.DistributionNotFound:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()


__all__ = ("CloudChannelServiceAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.channel_v1.types import common
from google.cloud.channel_v1.types import offers
from google.cloud.channel_v1.types import products
from google.protobuf import timestamp_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.channel.v1",
    manifest={
        "Entitlement",
        "Parameter",
        "AssociationInfo",
        "ProvisionedService",
        "CommitmentSettings",
        "RenewalSettings",
        "TrialSettings",
        "TransferableSku",
        "TransferEligibility",
    },
)


class Entitlement(proto.Message):
    r"""An entitlement is a representation of a customer's ability to
    use a service.

    Attributes:
        name (str):
            Output only. Resource name of an entitlement in the form:
            accounts/{account_id}/customers/{customer_id}/entitlements/{entitlement_id}.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time at which the
            entitlement is created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time at which the
            entitlement is updated.
        offer (str):
            Required. The offer resource name for which the entitlement
            is to be created. Takes the form:
            accounts/{account_id}/offers/{offer_id}.
        commitment_settings (google.cloud.channel_v1.types.CommitmentSettings):
            Commitment settings for a commitment-based
            Offer. Required for commitment based offers.
        provisioning_state (google.cloud.channel_v1.types.Entitlement.ProvisioningState):
            Output only. Current provisioning state of
            the entitlement.
        provisioned_service (google.cloud.channel_v1.types.ProvisionedService):
            Output only. Service provisioning details for
            the entitlement.
        suspension_reasons (Sequence[google.cloud.channel_v1.types.Entitlement.SuspensionReason]):
            Output only. Enumerable of all current
            suspension reasons for an entitlement.
        purchase_order_id (str):
            Optional. This purchase order (PO)
            information is for resellers to use for their
            company tracking usage. If a purchaseOrderId
            value is given, it appears in the API responses
            and shows up in the invoice. The property
            accepts up to 80 plain text characters.
        trial_settings (google.cloud.channel_v1.types.TrialSettings):
            Output only. Settings for trial offers.
        association_info (google.cloud.channel_v1.types.AssociationInfo):
            Association information to other
            entitlements.
        parameters (Sequence[google.cloud.channel_v1.types.Parameter]):
            Extended entitlement parameters. When
            creating an entitlement, valid parameters' names
            and values are defined in the offer's parameter
            definitions.
    """

    class ProvisioningState(proto.Enum):
        r"""Indicates the current provisioning state of the entitlement."""
        PROVISIONING_STATE_UNSPECIFIED = 0
        ACTIVE = 1
        SUSPENDED = 5

    class SuspensionReason(proto.Enum):
        r"""Suspension reason for an entitlement if
        [provisioning_state][google.cloud.channel.v1.Entitlement.provisioning_state]
        = SUSPENDED.
        """
        SUSPENSION_REASON_UNSPECIFIED = 0
        RESELLER_INITIATED = 1
        TRIAL_ENDED = 2
        RENEWAL_WITH_TYPE_CANCEL = 3
        PENDING_TOS_ACCEPTANCE = 4
        OTHER = 100

    name = proto.Field(proto.STRING, number=1,)
    create_time = proto.Field(proto.MESSAGE, number=5, message=timestamp_pb2.Timestamp,)
    update_time = proto.Field(proto.MESSAGE, number=6, message=timestamp_pb2.Timestamp,)
    offer = proto.Field(proto.STRING, number=8,)
    commitment_settings = proto.Field(
        proto.MESSAGE, number=12, message="CommitmentSettings",
    )
    provisioning_state = proto.Field(proto.ENUM, number=13, enum=ProvisioningState,)
    provisioned_service = proto.Field(
        proto.MESSAGE, number=16, message="ProvisionedService",
    )
    suspension_reasons = proto.RepeatedField(
        proto.ENUM, number=18, enum=SuspensionReason,
    )
    purchase_order_id = proto.Field(proto.STRING, number=19,)
    trial_settings = proto.Field(proto.MESSAGE, number=21, message="TrialSettings",)
    association_info = proto.Field(proto.MESSAGE, number=23, message="AssociationInfo",)
    parameters = proto.RepeatedField(proto.MESSAGE, number=26, message="Parameter",)


class Parameter(proto.Message):
    r"""Definition for extended entitlement parameters.
    Attributes:
        name (str):
            Name of the parameter.
        value (google.cloud.channel_v1.types.Value):
            Value of the parameter.
        editable (bool):
            Output only. Specifies whether this parameter is allowed to
            be changed. For example, for a Google Workspace Business
            Starter entitlement in commitment plan, num_units is
            editable when entitlement is active.
    """

    name = proto.Field(proto.STRING, number=1,)
    value = proto.Field(proto.MESSAGE, number=2, message=common.Value,)
    editable = proto.Field(proto.BOOL, number=3,)


class AssociationInfo(proto.Message):
    r"""Association links that an entitlement has to other
    entitlements.

    Attributes:
        base_entitlement (str):
            The name of the base entitlement, for which
            this entitlement is an add-on.
    """

    base_entitlement = proto.Field(proto.STRING, number=1,)


class ProvisionedService(proto.Message):
    r"""Service provisioned for an entitlement.
    Attributes:
        provisioning_id (str):
            Output only. Provisioning ID of the
            entitlement. For Google Workspace, this would be
            the underlying Subscription ID.
        product_id (str):
            Output only. The product pertaining to the
            provisioning resource as specified in the Offer.
        sku_id (str):
            Output only. The SKU pertaining to the
            provisioning resource as specified in the Offer.
    """

    provisioning_id = proto.Field(proto.STRING, number=1,)
    product_id = proto.Field(proto.STRING, number=2,)
    sku_id = proto.Field(proto.STRING, number=3,)


class CommitmentSettings(proto.Message):
    r"""Commitment settings for commitment-based offers.
    Attributes:
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Commitment start timestamp.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Commitment end timestamp.
        renewal_settings (google.cloud.channel_v1.types.RenewalSettings):
            Optional. Renewal settings applicable for a
            commitment-based Offer.
    """

    start_time = proto.Field(proto.MESSAGE, number=1, message=timestamp_pb2.Timestamp,)
    end_time = proto.Field(proto.MESSAGE, number=2, message=timestamp_pb2.Timestamp,)
    renewal_settings = proto.Field(proto.MESSAGE, number=4, message="RenewalSettings",)


class RenewalSettings(proto.Message):
    r"""Renewal settings for renewable Offers.
    Attributes:
        enable_renewal (bool):
            If false, the plan will be completed at the
            end date.
        resize_unit_count (bool):
            If true and enable_renewal = true, the unit (for example
            seats or licenses) will be set to the number of active units
            at renewal time.
        payment_plan (google.cloud.channel_v1.types.PaymentPlan):
            Describes how a reseller will be billed.
        payment_cycle (google.cloud.channel_v1.types.Period):
            Describes how frequently the reseller will be
            billed, such as once per month.
    """

    enable_renewal = proto.Field(proto.BOOL, number=1,)
    resize_unit_count = proto.Field(proto.BOOL, number=2,)
    payment_plan = proto.Field(proto.ENUM, number=5, enum=offers.PaymentPlan,)
    payment_cycle = proto.Field(proto.MESSAGE, number=6, message=offers.Period,)


class TrialSettings(proto.Message):
    r"""Settings for trial offers.
    Attributes:
        trial (bool):
            Determines if the entitlement is in a trial or not:

            -  ``true`` - The entitlement is in trial.
            -  ``false`` - The entitlement is not in trial.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            Date when the trial ends. The value is in milliseconds using
            the UNIX Epoch format. See an example `Epoch
            converter <https://www.epochconverter.com>`__.
    """

    trial = proto.Field(proto.BOOL, number=1,)
    end_time = proto.Field(proto.MESSAGE, number=2, message=timestamp_pb2.Timestamp,)


class TransferableSku(proto.Message):
    r"""TransferableSku represents information a reseller needs to
    view existing provisioned services for a customer that they do
    not own. Read-only.

    Attributes:
        transfer_eligibility (google.cloud.channel_v1.types.TransferEligibility):
            Describes the transfer eligibility of a SKU.
        sku (google.cloud.channel_v1.types.Sku):
            The SKU pertaining to the provisioning
            resource as specified in the Offer.
        legacy_sku (google.cloud.channel_v1.types.Sku):
            Optional. The customer to transfer has an
            entitlement with the populated legacy SKU.
    """

    transfer_eligibility = proto.Field(
        proto.MESSAGE, number=9, message="TransferEligibility",
    )
    sku = proto.Field(proto.MESSAGE, number=11, message=products.Sku,)
    legacy_sku = proto.Field(proto.MESSAGE, number=12, message=products.Sku,)


class TransferEligibility(proto.Message):
    r"""Specifies transfer eligibility of a SKU.
    Attributes:
        is_eligible (bool):
            Whether reseller is eligible to transfer the
            SKU.
        description (str):
            Localized description if reseller is not
            eligible to transfer the SKU.
        ineligibility_reason (google.cloud.channel_v1.types.TransferEligibility.Reason):
            Specified the reason for ineligibility.
    """

    class Reason(proto.Enum):
        r"""Reason of ineligibility."""
        REASON_UNSPECIFIED = 0
        PENDING_TOS_ACCEPTANCE = 1
        SKU_NOT_ELIGIBLE = 2
        SKU_SUSPENDED = 3

    is_eligible = proto.Field(proto.BOOL, number=1,)
    description = proto.Field(proto.STRING, number=2,)
    ineligibility_reason = proto.Field(proto.ENUM, number=3, enum=Reason,)


__all__ = tuple(sorted(__protobuf__.manifest))
